Package: leafdown
Type: Package
Title: Provides Drill Down Functionality for 'leaflet' Choropleths
Version: 1.2.0
Authors@R: c(
    person("Andreas", "Hofheinz", email = "andreas.hofheinz@outlook.com", role = "cre"),
    person("Peter", "Gandenberger", email = "peter.gandenberger@gmail.com", role = "aut"))
Description: Provides drill down functionality for 'leaflet' choropleths in 'shiny' apps.
License: GPL-3
Encoding: UTF-8
LazyData: true
Suggests: covr, testthat, knitr, rmarkdown, shinytest, dplyr,
        htmltools, raster, rmapshaper, shinycssloaders
RoxygenNote: 7.1.2
Depends: R (>= 3.5.0)
Imports: R6, leaflet, magrittr, checkmate, shiny, shinyjs
VignetteBuilder: knitr, shinycssloaders
NeedsCompilation: no
Packaged: 2022-09-12 19:28:03 UTC; andre
Author: Andreas Hofheinz [cre],
  Peter Gandenberger [aut]
Maintainer: Andreas Hofheinz <andreas.hofheinz@outlook.com>
Repository: CRAN
Date/Publication: 2022-09-19 06:06:11 UTC
Built: R 4.6.0; ; 2026-04-26 19:23:26 UTC; unix
